package org.richfaces;

import org.ajax4jsf.test.base.SeleniumTestBase;
import org.ajax4jsf.test.base.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class AjaxStatusTest extends SeleniumTestBase {

    public AjaxStatusTest() {
        super("http", "localhost", "8080");
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
        super.startSelenium(browser);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testAjaxStatusComponent() throws Exception {
        _testAjaxStatusComponent(Templates.DATATABLE);
    }

    private void _testAjaxStatusComponent(Templates template) {
        renderPage("/faces/pages/ajaxStatus/ajaxStatusTest.xhtml", template);

        String parentId = getParentId() + "_form:";
        String btn1 = parentId + "button1";
        clickById(btn1);

        String reg1Status = parentId + "region1:status";

        writeStatus("Ajax request is in progress");

        AssertVisible(reg1Status + ".start");
        AssertNotVisible(reg1Status + ".stop");
        String start = getTextById(reg1Status + ".start");
        Assert.assertTrue(start.startsWith("In progress..."));
        pause(3000, reg1Status);

        writeStatus("Ajax request completed");

        AssertNotVisible(reg1Status + ".start");
        AssertVisible(reg1Status + ".stop");
        String stop = getTextById(reg1Status + ".stop");
        Assert.assertTrue(stop.startsWith("Complete"));

        String btn2 = parentId + "button2";
        clickById(btn2);

        String reg2Status = parentId + "region2:status";

        writeStatus("Ajax request is in progress");

        AssertVisible(reg2Status + ".start");
        AssertNotVisible(reg2Status + ".stop");
        start = getTextById(reg2Status + ".start");
        Assert.assertTrue(start.startsWith("In progress..."));
        pause(3000, reg2Status);

        writeStatus("Ajax request completed");

        AssertNotVisible(reg2Status + ".start");
        AssertVisible(reg2Status + ".stop");
        stop = getTextById(reg1Status + ".stop");
        Assert.assertTrue(stop.startsWith("Complete"));

    }
}
