package org.richfaces;

import org.ajax4jsf.test.base.RichSeleniumTest;
import org.ajax4jsf.test.base.SeleniumTestBase;
import org.ajax4jsf.test.base.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class ProgressBarTest extends SeleniumTestBase implements
		RichSeleniumTest {

	public ProgressBarTest() {
		super("http", "localhost", "8080");
	}

	/**
	 * This method are invoking before selenium tests started
	 */
	@BeforeTest
	@Parameters( { "browser" })
	public void startSelenium(String browser) {
		super.startSelenium(browser);
	}

	/**
	 * This method are invoking after selenium tests completed
	 */
	@AfterTest
	public void stopSelenium() {
		super.stopSelenium();
	}

	@Test
	public void testProgressBarComponent() {
		_testProgressBarComponent(Templates.SIMPLE);
		_testProgressBarComponent(Templates.DATATABLE);
	}

	private void _testProgressBarComponent(Templates template) {
		
		renderPage(getTestUrl(), template);
	    
     	String parentId = getParentId() + "_form:";
    		
		String progressBarId = parentId + "progressBar1";
		int value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value < 0);
		String text = getTextById(progressBarId);
		writeStatus("Check if process not started");
		Assert.assertTrue(text.startsWith("Process not started"));
		
		writeStatus("Enable polling");
		enableProgressBar(progressBarId, true);
		
		pause(5000, progressBarId);
		
		writeStatus("Disable polling");
		enableProgressBar(progressBarId, false);
		
		writeStatus("Check label");
		text = getTextById(progressBarId + ":remain");
		Assert.assertTrue(text.endsWith("%"));
		
		writeStatus("Check value");
		value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value > 0);
		
		writeStatus("Enable polling");
		enableProgressBar(progressBarId, true);
		
		pause(5000, progressBarId);
		writeStatus("Check value");
		Assert.assertTrue((getProgressBarValue(progressBarId).intValue() != value));
		
		writeStatus("Disable polling");
		enableProgressBar(progressBarId, false);
		
		value = getProgressBarValue(progressBarId);
		pause(1500, progressBarId);
		writeStatus("Check value");
		Assert.assertTrue(getProgressBarValue(progressBarId) == value);
		
		clickById(parentId + "_complete");
		waitForAjaxCompletion();
		writeStatus("Check if process completed");
		text = getTextById(progressBarId);
		Assert.assertTrue(text.startsWith("Process completed"));
		
		
		// - Test client mode
		
		writeStatus("Check value");
		progressBarId = parentId + "progressBar2";
		value = getProgressBarValue(progressBarId);
		Assert.assertTrue(value == -5);
		
		text = getTextById(progressBarId);
		writeStatus("Check if process not started");
		Assert.assertTrue(text.startsWith("Process not started"));
		
		setProgressBarValue(progressBarId, 20);
		value = getProgressBarValue(progressBarId);
		writeStatus("Check value");
		Assert.assertTrue(value == 20);
		
		setProgressBarValue(progressBarId, 60);
		value = getProgressBarValue(progressBarId);
		writeStatus("Check value");
		Assert.assertTrue(value == 60);
		
		clickById(parentId + "_reset");
		waitForAjaxCompletion();
		
	}

	private void enableProgressBar(String id, boolean enable) {
		invokeFromComponent(id, (enable ? "enable" : "disable"), null);
	}

	private void setProgressBarValue(String id, Object value) {
		invokeFromComponent(id, "setValue", value);
	}

	private Integer getProgressBarValue(String id) {
		String value = invokeFromComponent(id, "getValue", null); //runScript(script.toString());
		Integer v = Integer.parseInt(value);
		return  v;
	}
	
    public String getTestUrl() {
		return "/faces/pages/progressBar/progressBarTest.xhtml";
	}


}
