/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces;

import org.ajax4jsf.test.base.RichSeleniumTest;
import org.ajax4jsf.test.base.SeleniumTestBase;
import org.ajax4jsf.test.base.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class SimpleTogglePanelTest  extends SeleniumTestBase implements RichSeleniumTest {
	
	public SimpleTogglePanelTest() {
		super("http", "localhost", "8080");
	}

	/**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
    	super.startSelenium(browser);
    }
    
    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
    	super.stopSelenium();
    }
    
    @Test
    public void testSimpleTogglePanelComponent() throws Exception {
    	_testSimpleTogglePanelComponent(Templates.SIMPLE);
    	_testSimpleTogglePanelComponent(Templates.DATATABLE);
    }
    
    private void _testSimpleTogglePanelComponent(Templates template) {
    	renderPage(getTestUrl(), template);
    	String parentId = getParentId() + "_form:";
    	String inputId = parentId + "_value";
        String outputId = parentId + "_value2";
        
    	String ajaxHeader = parentId + "panel1_header";
    	String serverHeader = parentId + "panel2_header";
    	String clientHeader = parentId + "panel3_header";
    	
    	String ajaxBody = parentId + "panel1_body";
    	String serverBody = parentId + "panel2_body";
    	String clientBody = parentId + "panel3_body";
    	
        clickById(clientHeader);
        Assert.assertTrue(isVisibleById(clientBody));
        clickById(clientHeader);
        Assert.assertFalse(isVisibleById(clientBody));
        
        clickById(ajaxHeader);
        waitForAjaxCompletion(5000);
        AssertValueEquals(inputId, "panel1");
        AssertTextEquals(outputId, "1");
        Assert.assertTrue(isVisibleById(ajaxBody));
        
        clickById(ajaxHeader);
        waitForAjaxCompletion(5000);
        AssertValueEquals(inputId, "panel1");
        AssertTextEquals(outputId, "2");
        Assert.assertFalse(isVisibleById(ajaxBody));
        
        clickCommandAndWait(serverHeader);
        AssertValueEquals(inputId, "panel2");
        AssertTextEquals(outputId, "3");
        Assert.assertTrue(isVisibleById(serverBody));
        
        clickCommandAndWait(serverHeader);
        AssertValueEquals(inputId, "panel2");
        AssertTextEquals(outputId, "4");
        Assert.assertFalse(isVisibleById(serverBody));
    }
    
    public String getTestUrl() {
		return "/faces/pages/simpleTogglePanel/simpleTogglePanel.xhtml";
	}


}
